#! /bin/bash
#
# bootstrap
# General-purpose home directory bootstrap script
# Copyright (C) 2020 Vintage Salt <rehashedsalt@cock.li>
#
# Distributed under terms of the MIT license.
#
set -e

# Read-only set-once variables
declare -r _name="$(basename -- "$0")"
# Options
declare -i _opthelp
declare -i _optverbose
declare _optdest="$HOME"
declare _optgitdir=".dotfiles"
declare _optrepo="git@git.9iron.club:salt/home"
# Working variables
declare -a _args
declare _return

# Helper functions
log() {
	# Print a line to the terminal if _optverbose is greater than $2
	# $2 defaults to 0
	# loglevel 0: Daily-use messages
	# loglevel 1: Detailed but not quite debugging
	# loglevel 2: Definitely debugging
	[ -z "$1" ] && return 1
	if (( _optverbose >= ${2:-0} )); then
		printf "%s\\n" "$1"
	fi
}
warn() {
	# Print a yellow line to the terminal, respecting _optverbose
	[ -z "$1" ] && return 1
	if (( _optverbose >= ${2:-0} )); then
		if [ -t 1 ]; then
			printf "\\e[33m%s\\e[0m\\n" "$1"
		else
			printf "WARN: %s\\n" "$1"
		fi
	fi
}
error() {
	# Print a red line to the terminal, exit if $2 is specified
	[ -z "$1" ] && return 1
	if [ -t 2 ]; then
		printf "\\e[31m%s\\e[0m\\n" "$1" 1>&2
	else
		printf "ERROR: %s\\n" "$1" 1>&2
	fi
	[ -z "$2" ] && return
	exit "${2:-1}"
}
has() {
	# Parse out all arguments and try to find them in path
	# If an argument cannot be found, set _return and fail
	for prog in "$@"; do
		if ! command -v "$prog" > /dev/null 2>&1; then
			_return="$prog"
			return 1
		fi
	done
	return 0
}

# Helper functions
cleanup() {
	[ -z "$1" ] && warn "No argument passed to cleanup"
	rm -rf "$1" || warn "Failed to remove $1: $?"
}

# Core program functions
printhelp() {
	cat << EOF
Usage: $_name [OPTION]...

  -d			Bootstrap directory. Defaults to ~
  -h			Print this help text
  -r			Repository. Defaults to git@git.9iron.club:salt/home
  -v			Print more status messages. Stacks

Copyright (c) 2020 rehashedsalt@cock.li
Licensed under the MIT license
EOF
}
build-home() {
	# Build a home directory
	local repo="$_optrepo"
	local dest="$_optdest"
	log "Using remote repository $repo" 2
	log "Using destination $dest" 2

	# Build a skeleton folder layout
	log "Building skeleton folder layout"
	pushd "$dest" > /dev/null 2>&1
	mkdir Desktop Documents Downloads Music Pictures Projects Public Templates Videos || warn "Error creating skeleton folder layout"
	chmod 775 Public || warn "Error making directory Public world-accessible"
	popd > /dev/null 2>&1

	# Clone our repo into a temporary directory
	log "Cloning repository into temporary directory"
	local tmpdir="$(mktemp -d "tmp.$USER.$_name.XXXXXXXX" -p "/tmp")"
	trap "cleanup $tmpdir" EXIT
	pushd "$tmpdir" > /dev/null 2>&1
	log "Cloning into temporary directory: $tmpdir" 2
	git clone \
		--recursive \
		--depth 50 \
		--separate-git-dir="./$_optgitdir" \
		"$repo" \
		"$tmpdir" > /dev/null 2>&1
	git config \
		status.showUntrackedFiles no \
		--git-dir="./$_optgitdir"
	popd > /dev/null 2>&1

	# Sync the temporary directory over
	log "Moving cloned repository into destination"
	log "Moving from $tmpdor to $dest"
	rsync -rl "$tmpdir/" "$dest/"
}

# Main
main() {
	# Parse out arguments
	while [ -n "$1" ]; do
		# Parse out flags
		while getopts ":c:d:hr:v" opt; do
			case $opt in
				d)
					_optdest="$OPTARG"
					;;
				h)
					_opthelp=1
					;;
				r)
					_optrepo="$OPTARG"
					;;
				v)
					_optverbose+=1
					;;
				:)
					error "Option requires argument: -$OPTARG" 2
					;;
				*)
					error "Invalid option: -$OPTARG" 2
					;;
			esac
		done
		# Store arguments
		shift $((OPTIND - 1))
		if [ -n "$1" ]; then
			_args+=("$1")
			shift
		fi
		unset OPTIND
	done
	# Early hook for help
	[ -n "$_opthelp" ] && printhelp && exit 0
	# Validate critical options
	# TODO: That
	# Validate core program dependencies
	log "Validating dependencies" 2
	if ! has basename git mktemp rsync; then
		error "Failed to find program: $_return" 1
	fi

	# Do the do
	build-home
	exit 0
}

main "$@"

